import 'dart:convert';
import 'dart:math';
import 'package:flutter_test/flutter_test.dart';

import 'package:odds_account_sdk/client.dart';

String getRandomString(length) {
  const chars =
      'AaBbCcDdEeFfGgHhIiJjKkLlMmNnOoPpQqRrSsTtUuVvWwXxYyZz1234567890';
  Random rnd = Random();

  return String.fromCharCodes(Iterable.generate(
      length, (_) => chars.codeUnitAt(rnd.nextInt(chars.length))));
}

void main() async {
  final birthdate = DateTime.utc(2022, 6, 22, 14, 55, 0);
  final address = Address("CN", "ZJ", "HZ", "CangQian");
  final account = Account(
    '00001',
    "test.com",
    "public",
    DateTime.now(),
    'bob',
    '123456',
    'bob@example.com',
    'bob',
    'given',
    'family',
    'middle',
    'avatar.png',
    Gender.unknown,
    birthdate,
    address,
  );

  OddsAccountClient client = OddsAccountClient.authing(
      '62b51f95688cd8c27d2b328e', '62b5309cbfdfbcdaa19d7acb');
  await client.init();

  test('serialize account to json', () {
    final json = jsonEncode(account);

    expect(
        json,
        allOf(
            contains('"user_id":"00001"'),
            contains('"gender":"U"'),
            contains('"birthdate":"2022-06-22T14:55:00.000Z"'),
            contains('"country":"CN"')));
  });

  test('serialize result to json', () {
    final success = Result.success(account);
    final successJson = jsonEncode(success);

    expect(
        successJson,
        allOf(
          contains('"code":200'),
          contains('"message":""'),
          contains('"user_id":"00001"'),
        ));

    final failure = Result.failure(StatusCode.notFound, "User not exists");
    final failureJson = jsonEncode(failure);
    expect(
        failureJson,
        allOf(
          contains('"code":404'),
          contains('"message":"User not exists"'),
        ));
  });

  test('login by username and password', () async {
    final r1 = await client.loginByPassword("test", "incorrect password");
    expect(r1.isSuccess(), false);

    final r2 = await client.loginByPassword("test", "OLt4C34aRvHBP4ITPlcf");
    expect(r2.isSuccess(), true);
  });

  test('update brief', () async {
    final r1 = await client.loginByPassword("test", "OLt4C34aRvHBP4ITPlcf");
    expect(r1.isSuccess(), true);

    final randomBrief = getRandomString(16);
    final r2 = await client.updateBrief(randomBrief);
    expect(r2.isSuccess(), true);

    await client.logout();

    final r3 = await client.loginByPassword("test", "OLt4C34aRvHBP4ITPlcf");
    expect(r3.isSuccess(), true);
    expect(r3.data!.account.brief, randomBrief);
  });

  test('upload avatar', () async {
    final r = await client.uploadAvatar(r'assets/man.png');
    expect(r.isSuccess(), true);
    expect(
        r.data,
        allOf(contains('https://files.authing.co/user-contents/avatar'),
            contains('.png')));
  });

  test('update user profile', () async {
    final r1 = await client.loginByPassword("test", "OLt4C34aRvHBP4ITPlcf");
    expect(r1.data?.account, isNotNull);

    final String randomNickname = getRandomString(8);
    final String randomGivenName = getRandomString(10);
    final String randomFamilyName = getRandomString(10);
    final String randomMiddleName = getRandomString(10);
    final DateTime twentyYearBefore =
        DateTime.now().subtract(const Duration(days: 365 * 20));
    final Address randomAddress =
        AddressBuilder.clone(r1.data!.account.address!)
            .withCountry(getRandomString(6))
            .withProvince(getRandomString(6))
            .withCity(getRandomString(6))
            .withStreet(getRandomString(20))
            .build();
    final Gender randomGender =
        randomNickname[0].toUpperCase() == randomNickname[0]
            ? Gender.male
            : Gender.female;

    final r2 = await client.uploadAvatar(
        randomGender == Gender.male ? r'assets/man.png' : r'assets/woman.png');
    expect(r2.isSuccess(), true);

    final Account account = AccountBuilder.clone(r1.data!.account)
        .withNickname(randomNickname)
        .withGivenName(randomGivenName)
        .withFamilyName(randomFamilyName)
        .withMiddleName(randomMiddleName)
        .withBirthdate(twentyYearBefore)
        .withGender(randomGender)
        .withPhoto(r2.data!)
        .withAddress(randomAddress)
        .build();

    final r3 = await client.updateProfile(account);

    expect(r3.isSuccess(), true);

    expect(r3.data!.userId, r1.data!.account.userId);
    expect(r3.data!.username, r1.data!.account.username);
    expect(r3.data!.phone, r1.data!.account.phone);
    expect(r3.data!.email, r1.data!.account.email);

    expect(r3.data!.nickname, randomNickname);
    expect(r3.data!.givenName, randomGivenName);
    expect(r3.data!.familyName, randomFamilyName);
    expect(r3.data!.middleName, randomMiddleName);
    expect(r3.data!.birthdate, twentyYearBefore);
    expect(r3.data!.gender, randomGender);
    expect(r3.data!.photo, r2.data);
    expect(r3.data!.address!.country, randomAddress.country);
    expect(r3.data!.address!.province, randomAddress.province);
    expect(r3.data!.address!.city, randomAddress.city);
    expect(r3.data!.address!.street, randomAddress.street);

    final r4 = await client.loginByPassword("test", "OLt4C34aRvHBP4ITPlcf");
    expect(r4.data?.account, isNotNull);

    expect(r4.data!.account.userId, r1.data!.account.userId);
    expect(r4.data!.account.username, r1.data!.account.username);
    expect(r4.data!.account.phone, r1.data!.account.phone);
    expect(r4.data!.account.email, r1.data!.account.email);

    expect(r4.data!.account.nickname, randomNickname);
    expect(r4.data!.account.givenName, randomGivenName);
    expect(r4.data!.account.familyName, randomFamilyName);
    expect(r4.data!.account.middleName, randomMiddleName);
    expect(r4.data!.account.birthdate, twentyYearBefore);
    expect(r4.data!.account.gender, randomGender);
    expect(r4.data!.account.photo, r2.data);
    expect(r4.data!.account.address!.country, randomAddress.country);
    expect(r4.data!.account.address!.province, randomAddress.province);
    expect(r4.data!.account.address!.city, randomAddress.city);
    expect(r4.data!.account.address!.street, randomAddress.street);
  });

  test('claim code for updateEmail', () async {
    final r1 = await client.loginByPassword("test", "OLt4C34aRvHBP4ITPlcf");
    expect(r1.isSuccess(), true, reason: r1.message);

    final r2 = await client.claimUpdateEmailCode('youyoubang@jlsj32.wecom.work');
    expect(r2.isSuccess(), true, reason: r2.message);

    // final r3 = await client.claimUpdateEmailCode('email2');
    // expect(r3.isSuccess(), true, reason: r3.message);
  });

  test('bind email', () async {
    final r1 = await client.loginByPassword("test", "OLt4C34aRvHBP4ITPlcf");
    expect(r1.isSuccess(), true, reason: r1.message);

    final r2 = await client.bindEmail('youyoubang@jlsj32.wecom.work', '604913');
    expect(r2.isSuccess(), true, reason: r2.message);
  });

  test('update email', () async {
    final r1 = await client.loginByPassword("test", "OLt4C34aRvHBP4ITPlcf");
    expect(r1.isSuccess(), true, reason: r1.message);

    final r2 = await client.updateEmail(
        'email', 'emailCode', 'oldEmail', 'oldEmailCode');
    expect(r2.isSuccess(), true, reason: r2.message);
  });

  test('unbind email', () async {
    final r1 = await client.loginByPassword("test", "OLt4C34aRvHBP4ITPlcf");
    expect(r1.isSuccess(), true, reason: r1.message);

    final r2 = await client.unbindEmail();
    expect(r2.isSuccess(), true, reason: r2.message);
  });

  test('claim phone otp', () async {
    final r1 = await client.loginByPassword("test", "OLt4C34aRvHBP4ITPlcf");
    expect(r1.isSuccess(), true, reason: r1.message);

    final r2 = await client.claimPhoneOtp('17600370166');
    expect(r2.isSuccess(), true, reason: r2.message);
  });

  test('bind phone', () async {
    final r1 = await client.loginByPassword("test", "OLt4C34aRvHBP4ITPlcf");
    expect(r1.isSuccess(), true, reason: r1.message);

    final r2 = await client.bindPhone('phoneNumber', 'phoneOtp');
    expect(r2.isSuccess(), true, reason: r2.message);
  });

  test('update phone', () async {
    final r1 = await client.loginByPassword("test", "OLt4C34aRvHBP4ITPlcf");
    expect(r1.isSuccess(), true, reason: r1.message);

    final r2 = await client.updatePhone(
        'phoneNumber', 'phoneOtp', 'oldPhoneNumber', 'oldPhoneOtp');
    expect(r2.isSuccess(), true, reason: r2.message);
  });

  test('unbind phone', () async {
    final r1 = await client.loginByPassword("test", "OLt4C34aRvHBP4ITPlcf");
    expect(r1.isSuccess(), true, reason: r1.message);

    final r2 = await client.unbindPhone();
    expect(r2.isSuccess(), true, reason: r2.message);
  });

  test('update username', () async {
    final r1 = await client.loginByPassword("test", "OLt4C34aRvHBP4ITPlcf");
    expect(r1.isSuccess(), true, reason: r1.message);

    final r2 = await client.updateUsername('newName');
    expect(r2.isSuccess(), true, reason: r2.message);
  });
}
